@@ -22,6 +22,7 @@
 //! use datasize::data_size;
 //!
 //! let data: Vec<u64> = vec![1, 2, 3];
+//! #[cfg(feature = "std")]
 //! assert_eq!(data_size(&data), 24);
 //! ```
 //!
@@ -33,6 +34,7 @@
 //! ```rust
 //! use datasize::DataSize;
 //!
+//! #[cfg(feature = "std")]
 //! // A `Vec` of any kind may have elements added or removed, so it changes size.
 //! assert!(Vec::<u64>::IS_DYNAMIC);
 //!
@@ -49,7 +51,9 @@
 //! ```rust
 //! use datasize::DataSize;
 //!
+//! #[cfg(feature = "std")]
 //! assert_eq!(Box::<u64>::STATIC_HEAP_SIZE, 8);
+//! #[cfg(feature = "std")]
 //! assert!(!Box::<u64>::IS_DYNAMIC);
 //! ```
 //!
@@ -71,6 +75,7 @@
 //!     value.len() * 512
 //! }
 //!
+//! #[cfg(feature = "std")]
 //! #[derive(DataSize)]
 //! struct MyStruct {
 //!     items: Vec<u32>,
@@ -94,6 +99,7 @@
 //!     counter: Box<u64>,
 //! }
 //!
+//! #[cfg(feature = "std")]
 //! impl DataSize for MyType {
 //!     // `MyType` contains a `Vec`, so `IS_DYNAMIC` is set to true.
 //!     const IS_DYNAMIC: bool = true;
@@ -114,6 +120,7 @@
 //!     counter: Box::new(42),
 //! };
 //!
+//! #[cfg(feature = "std")]
 //! // Three i64 and one u64 on the heap sum up to 32 bytes:
 //! assert_eq!(data_size(&my_data), 32);
 //! ```
@@ -124,17 +131,20 @@
 //! ```
 //! # use datasize::{DataSize, data_size};
 //! // Equivalent to the manual implementation above:
+//! #[cfg(feature = "std")]
 //! #[derive(DataSize)]
 //! struct MyType {
 //!     items: Vec<i64>,
 //!     flag: bool,
 //!     counter: Box<u64>,
 //! }
+//! # #[cfg(feature = "std")]
 //! # let my_data = MyType {
 //! #     items: vec![1, 2, 3],
 //! #     flag: true,
 //! #     counter: Box::new(42),
 //! # };
+//! # #[cfg(feature = "std")]
 //! # assert_eq!(data_size(&my_data), 32);
 //! ```
 //!
@@ -182,6 +192,16 @@
 //! on the standard library. This can be used to derive the `DataSize` trait for types without
 //! boilerplate, even though their heap size will usually be 0.
 //!
+//! ## Arrays and const generics
+//!
+//! By default, this crate requires at least Rust version 1.51.0, in order to implement DataSize
+//! for [T; N] arrays generically. This implementation is provided by the "const-generics"
+//! feature flag, which is enabled by default. In order to use an older Rust version,
+//! you can specify [`default-features = false`](https://doc.rust-lang.org/cargo/reference/features.html#dependency-features) and `features = ["std"]` for `datasize` in your Cargo.toml.
+//!
+//! When the `const-generics` feature flag is disabled, a DataSize implementation will be provided
+//! for arrays of small sizes, and for some larger sizes related to powers of 2.
+//!
 //! ## Known issues
 //!
 //! The derive macro currently does not support generic structs with inline type bounds, e.g.
@@ -329,6 +349,7 @@ macro_rules! tuple_heap_size {
     };
 }
 
+#[cfg(not(feature = "const-generics"))]
 macro_rules! array_heap_size {
     ($($n:tt)+) => {
         $(
@@ -388,8 +409,28 @@ where
     }
 }
 
+#[cfg(not(feature = "const-generics"))]
 array_heap_size!(0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 128 192 256 384 512 1024 2048 4096 8192 16384 1048576 2097152 3145728 4194304);
 
+#[cfg(feature = "const-generics")]
+impl<T, const N: usize> DataSize for [T; N]
+where
+    T: DataSize,
+{
+    const IS_DYNAMIC: bool = T::IS_DYNAMIC;
+
+    const STATIC_HEAP_SIZE: usize = T::STATIC_HEAP_SIZE * N;
+
+    #[inline]
+    fn estimate_heap_size(&self) -> usize {
+        if T::IS_DYNAMIC {
+            self[..].iter().map(DataSize::estimate_heap_size).sum()
+        } else {
+            T::STATIC_HEAP_SIZE * N
+        }
+    }
+}
+
 // REFERENCES
 
 impl<T> DataSize for &T {
@@ -524,25 +565,39 @@ mod tests {
         assert_eq!(Result::<u8, u16>::STATIC_HEAP_SIZE, 0);
         assert!(!Result::<u8, u16>::IS_DYNAMIC);
 
+        #[cfg(feature = "std")]
         assert_eq!(Result::<u8, Box<u16>>::STATIC_HEAP_SIZE, 0);
+        #[cfg(feature = "std")]
         assert!(Result::<u8, Box<u16>>::IS_DYNAMIC);
 
+        #[cfg(feature = "std")]
         assert_eq!(Result::<Box<u8>, u16>::STATIC_HEAP_SIZE, 0);
+        #[cfg(feature = "std")]
         assert!(Result::<Box<u8>, u16>::IS_DYNAMIC);
 
+        #[cfg(feature = "std")]
         assert_eq!(Result::<Box<u8>, Box<u16>>::STATIC_HEAP_SIZE, 1);
+        #[cfg(feature = "std")]
         assert!(Result::<Box<u8>, Box<u16>>::IS_DYNAMIC);
 
+        #[cfg(feature = "std")]
         assert_eq!(Result::<Box<u16>, Box<u16>>::STATIC_HEAP_SIZE, 2);
+        #[cfg(feature = "std")]
         assert!(!Result::<Box<u16>, Box<u16>>::IS_DYNAMIC);
 
+        #[cfg(feature = "std")]
         assert_eq!(Result::<u16, Vec<u16>>::STATIC_HEAP_SIZE, 0);
+        #[cfg(feature = "std")]
         assert!(Result::<u16, Vec<u16>>::IS_DYNAMIC);
 
+        #[cfg(feature = "std")]
         assert_eq!(Result::<Vec<u16>, u16>::STATIC_HEAP_SIZE, 0);
+        #[cfg(feature = "std")]
         assert!(Result::<Vec<u16>, u16>::IS_DYNAMIC);
 
+        #[cfg(feature = "std")]
         assert_eq!(Result::<Vec<u16>, Vec<u16>>::STATIC_HEAP_SIZE, 0);
+        #[cfg(feature = "std")]
         assert!(Result::<Vec<u16>, Vec<u16>>::IS_DYNAMIC);
     }
 
